import { Redis } from '@upstash/redis';
import { Run } from '../run/types';
export declare const RegisteredLogger: {
    readonly AGENT: "AGENT";
    readonly WORKFLOW: "WORKFLOW";
    readonly LLM: "LLM";
};
export type RegisteredLogger = (typeof RegisteredLogger)[keyof typeof RegisteredLogger];
export declare const LogLevel: {
    readonly DEBUG: "DEBUG";
    readonly INFO: "INFO";
    readonly WARN: "WARN";
    readonly ERROR: "ERROR";
    readonly NONE: "NONE";
};
export type LogLevel = (typeof LogLevel)[keyof typeof LogLevel];
export interface BaseLogMessage extends Run {
    message: string;
    destinationPath: string;
    type: RegisteredLogger;
}
export interface Logger<T extends BaseLogMessage = BaseLogMessage> {
    debug(message: T | string, ...args: any[]): void | Promise<void>;
    info(message: T | string, ...args: any[]): void | Promise<void>;
    warn(message: T | string, ...args: any[]): void | Promise<void>;
    error(message: T | string, ...args: any[]): void | Promise<void>;
    cleanup?(): Promise<void>;
}
type ConsoleLoggerConfig = {
    type: 'CONSOLE';
    level?: LogLevel;
};
type FileLoggerConfig = {
    type: 'FILE';
    level?: LogLevel;
    dirPath?: string;
};
type UpstashLoggerConfig = {
    type: 'UPSTASH';
    level?: LogLevel;
    url: string;
    token: string;
    key?: string;
};
type LoggerConfig = ConsoleLoggerConfig | FileLoggerConfig | UpstashLoggerConfig;
type LoggerTypeMap = {
    CONSOLE: ConsoleLogger<BaseLogMessage>;
    FILE: FileLogger<BaseLogMessage>;
    UPSTASH: UpstashRedisLogger<BaseLogMessage>;
};
export declare abstract class BaseLogger<T extends BaseLogMessage = BaseLogMessage> implements Logger<T> {
    protected level: LogLevel;
    constructor(level?: LogLevel);
    abstract log(level: LogLevel, message: T | string, ...args: any[]): void | Promise<void>;
    debug(message: T | string, ...args: any[]): void | Promise<void>;
    info(message: T | string, ...args: any[]): void | Promise<void>;
    warn(message: T | string, ...args: any[]): void | Promise<void>;
    error(message: T | string, ...args: any[]): void | Promise<void>;
    protected formatMessage(message: T | string): string;
    protected formatLogEntry(level: LogLevel, message: T | string): any;
    getLogsByRunId(runId: string): Promise<T[]>;
}
export declare class ConsoleLogger<T extends BaseLogMessage = BaseLogMessage> extends BaseLogger<T> {
    constructor(level?: LogLevel);
    log(level: LogLevel, message: T | string, ...args: any[]): void;
}
export declare class FileLogger<T extends BaseLogMessage = BaseLogMessage> extends BaseLogger<T> {
    #private;
    constructor(dirPath?: string, level?: LogLevel);
    log(level: LogLevel, message: T): void;
    private ensureDirectoryExists;
    private writeLogToFile;
}
export declare class UpstashRedisLogger<T extends BaseLogMessage = BaseLogMessage> extends BaseLogger<T> {
    #private;
    constructor(redis: Redis, key?: string, level?: LogLevel);
    log(level: LogLevel, message: T): Promise<void>;
    getLogs(): Promise<string[]>;
    deleteLogsByKey(key: string): Promise<void>;
    getLogsByRunId(runId: string): Promise<T[]>;
    cleanup(): Promise<void>;
}
export declare class MultiLogger<T extends BaseLogMessage = BaseLogMessage> implements Logger<T> {
    private loggers;
    constructor(loggers: Logger<T>[]);
    debug(message: T | string, ...args: any[]): Promise<void>;
    info(message: T | string, ...args: any[]): Promise<void>;
    warn(message: T | string, ...args: any[]): Promise<void>;
    error(message: T | string, ...args: any[]): Promise<void>;
    cleanup(): Promise<void>;
}
export declare const createLogger: <Type extends LoggerConfig["type"], T extends BaseLogMessage = BaseLogMessage>(config: Extract<LoggerConfig, {
    type: Type;
}>) => LoggerTypeMap[Type];
export declare function createMultiLogger<T extends BaseLogMessage = BaseLogMessage>(loggers: Logger<T>[]): Logger<T>;
export declare const noopLogger: {
    debug: () => void;
    info: () => void;
    warn: () => void;
    error: () => void;
    cleanup: () => void;
};
export {};
